/*
 *      Sdland.c                       1.006                   23.04.2011
 *
 *      Primzahl-Berechnung nach Norbert Sdland in einem iterierten
 *      Verfahren.
 *
 *      Literatur:
 *      [0397Bibel]
 *      "Die Bibel, oder die ganze Heilige Schrift Alten und Neuen Testaments
 *      nach der deutschen Uebersetzung Dr. Martin Luthers.", Revision durch
 *      Dr. Johann Philipp Fresenius, (1751); Druck und Verlag von Heinrich
 *      Ludwig Brnner, Frankfurt am Main, 40. Auflage, (1841); der Text gilt
 *      als vollstndig abgeschlossen seit dem Konzil von Karthago, (397)
 *
 *      [1995OliS]
 *      Olivastro D., Schmidt M.: Das chinesische Dreieck (Ancient Puzzles),
 *      Lizenzausgabe mit freundlicher Genehmigung des Droemer Knaur
 *      Verlages, Mnchen fr Zweitausendeins, Frankfurt am Main, (1995)
 *
 *      Bearbeitung:
 *      30.03.2011 - 23.04.2011 Norbert Sdland, D-73431 Aalen
 */

/*
 *      Zugang:
 *      2 ist nur durch 2 und 1 teilbar, also eine Primzahl.
 *      Eine Zahl ist durch 2 teilbar, wenn sie gerade ist.
 *      Als mgliche weitere Primzahlen kommen somit nur ungerade Zahlen,
 *      die grer als 2 sind, in Frage.
 *
 *      Eine Zahl ist dann eine Primzahl, wenn sie nicht durch alle voran
 *      gegangenen Primzahlen teilbar ist, deren Quadrat kleiner als die
 *      zu untersuchende Zahl ist.
 *
 *      Nach 5.Mose 19,15 ist ein richtiges Ergebnis auf zwei oder drei
 *      unabhngigen Rechenwegen erhltlich. Nach 3.Mose 19,14 muss auch
 *      der Rechenweg offengelegt werden.
 *
 *      Auer dem Ergebnis wird auch die bentigte Rechenzeit ausgegeben.
 */

#include        <SOBS.Kpf>
#include        <StdEA.Kpf>
#include        <StdBib.Kpf>


/**********************************************************/
  Nichts         Hauptteil( Ganzzahl     ParameterAnzahl ,
                            Buchstabe   *Parameter[ ] )
/**********************************************************/
{
  Buchstabe              ErgebnisDateiName[ ] = "Sdland.csv" ,
                         PrimzahlDateiName[ ] = "Sdland.dmp" ,
                        *AusgabeFormat[ ] = {
                           "%10lu.Ergebnis =  %10lu: %10lu Schleifen\n" ,
                           "%10lu.Ergebnis =  %10lu: %10lu Schleife\n"
                         } ;
  NatuerlicheZahl        pb ;
  GrosseNatuerlicheZahl  z , n , n_max , p , s , ss ;
  Datei                 *ErgebnisDatei , *PrimzahlDatei ;
  ZeitStruktur           Zeit ;
  ErgebnisGrosseGanzzahlDivision Probe ;

  /* Vorbereitung: */
  /*---------------*/
  Wenn( ParameterAnzahl  >  1 )
    n_max = TextZuNatuerlicherZahl( Parameter[ 1 ] , NullZeiger , 10 ) ;
  Sonst
    n_max = 0 ;
  Wenn( !n_max )
    BildAusgabe( "Syntax:\nSdland [AnzahlPrimzahlen]\n\n" ) ;
  pb = PlatzBedarf( GrosseNatuerlicheZahl ) ;
  SetzenWannStrgC( 1 ) ;

  /* Primzahlen erzeugen und ausgeben: */
  /*-----------------------------------*/
  ErgebnisDatei = DateiOeffnen( ErgebnisDateiName , "w+t" ) ;
  Wenn( !ErgebnisDatei )
    BildAusgabe( "Fehler beim ffnen der Datei \"%s\".\n\7" ,
                 ErgebnisDateiName ) ;
  PrimzahlDatei = DateiOeffnen( PrimzahlDateiName , "w+b" ) ;
  Wenn( !PrimzahlDatei )
    BildAusgabe( "Fehler beim ffnen der Datei \"%s\".\n\7" ,
                 PrimzahlDateiName ) ;

    ZeitAbfragen( &Zeit ) ;
    DateiAusgabe(
      ErgebnisDatei ,
      "%02u:%02u:%02u:%02u\n" ,
      Zeit.Zeit_Stunden ,
      Zeit.Zeit_Minuten ,
      Zeit.Zeit_Sekunden ,
      Zeit.Zeit_HundertstelSekunden
    ) ;
    Wenn( n_max  >  0 ) {
      n = 1 ;
      z = 2 ;
      s = 1 ;
      InDateiSchreiben( &z , pb , 1 , PrimzahlDatei ) ;
      DateiAusgabe( ErgebnisDatei , "%lu,%lu,%lu\n" , n , z , s ) ;
      BildAusgabe( AusgabeFormat[ s  ==  1 ? 1 : 0 ] , n , z , s ) ;
    }
    ZaehlSchleife( n = 1 , z = 3 , ss = 0 ; n  <  n_max ; z+= 2 ) {
      ZaehlSchleife( s = 1 , p = 2 ; s  <  n   &&   p * p  <  z ; s++ ) {
        DateiPositionSetzen( PrimzahlDatei ,  s * pb , SetzeAbAnfang ) ;
        VonDateiLesen( &p , pb , 1 , PrimzahlDatei ) ;
        Probe = GrosseGanzzahlDivision( z , p ) ;
        Wenn( !Probe.GanzzahlDivisionsRest ) {
          ss += s ;
          s = n + 1 ;                   /* keine Primzahl */
        }
      }
      Wenn( s  <=  n ) {                /* neue Primzahl gefunden */
        DateiPositionSetzen( PrimzahlDatei , 0 , SetzeAbEnde ) ;
        InDateiSchreiben( &z , pb , 1 , PrimzahlDatei ) ;
        n++ ;                           /* Anzahl der Primzahl erhhen */
        ss += s ;
        DateiAusgabe( ErgebnisDatei , "%lu,%lu,%lu\n" , n , z , ss ) ;
        BildAusgabe( AusgabeFormat[ ss  ==  1 ? 1 : 0 ] , n , z , ss ) ;
        ss = 0 ;                        /* nchste Primzahl suchen */
      }
    }
    ZeitAbfragen( &Zeit ) ;
    DateiAusgabe(
      ErgebnisDatei ,
      "%02u:%02u:%02u:%02u\n" ,
      Zeit.Zeit_Stunden ,
      Zeit.Zeit_Minuten ,
      Zeit.Zeit_Sekunden ,
      Zeit.Zeit_HundertstelSekunden
    ) ;
  AlleDateienSchliessen( ) ;
}
/*__ Ende von Hauptteil __*/

/*
 * Ende von "Sdland.c"
 */
